From 5ec5ac3a329bc54ec8c60c70c8bf5602bd0cec37 Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@free.fr>
Date: Mon, 22 Feb 2010 17:20:27 +0100
Subject: [PATCH] input: Fill in the input_item es.

Instead of using es_format_t we use a custom structure to limit the memory footprint.
This allow libvlc_media_get_es() to properly work after libvlc_media_parse().
---
 include/vlc_input_item.h |   32 ++++++++++++++++++++++++++++
 src/control/media.c      |   12 +++++-----
 src/input/es_out.c       |    4 +++
 src/input/item.c         |   52 ++++++++++++++++++++++++++++++++++++++++++++++
 src/input/item.h         |    1 +
 5 files changed, 95 insertions(+), 6 deletions(-)

diff --git a/include/vlc_input_item.h b/include/vlc_input_item.h
index d4858c9..7cc22d2 100644
--- a/include/vlc_input_item.h
+++ b/include/vlc_input_item.h
@@ -51,6 +51,35 @@ struct info_category_t
     struct info_t **pp_infos;     /**< Pointer to an array of infos */
 };
 
+/* Stripped down version of es_format_t because es_format_t can become very
+ * large in term of memory. This is mostly libvlc_media_es_t */
+typedef struct input_item_track_info_t
+{
+    vlc_fourcc_t i_codec;
+    int          i_id;
+    int          i_cat; /* @see es_format_category_e */
+
+    /* Codec specific */
+    int         i_profile;
+    int         i_level;
+
+    union
+    {
+        struct
+        {
+            /* Video specific */
+            unsigned    i_height;
+            unsigned    i_width;
+        } video;
+        struct
+        {
+            /* Audio specific */
+            unsigned    i_channels;
+            unsigned    i_rate;
+        } audio;
+    } u;
+} input_item_track_info_t;
+
 struct input_item_t
 {
     VLC_GC_MEMBERS
@@ -73,6 +102,9 @@ struct input_item_t
     int         i_es;                /**< Number of es format descriptions */
     es_format_t **es;                /**< Es formats */
 
+    int i_tracks_info;               /**< Number of track info descriptions */
+    input_item_track_info_t **tracks_info;/**< Tracks Info */
+
     input_stats_t *p_stats;          /**< Statistics */
     int           i_nb_played;       /**< Number of times played */
 
diff --git a/src/control/media.c b/src/control/media.c
index 1e8675b..e0d392b 100644
--- a/src/control/media.c
+++ b/src/control/media.c
@@ -688,7 +688,7 @@ libvlc_media_get_tracks_info( libvlc_media_t *p_md, libvlc_media_track_info_t **
     input_item_t *p_input_item = p_md->p_input_item;
     vlc_mutex_lock( &p_input_item->lock );
 
-    const int i_es = p_input_item->i_es;
+    const int i_es = p_input_item->i_tracks_info;
     *pp_es = (i_es > 0) ? malloc( i_es * sizeof(libvlc_media_track_info_t) ) : NULL;
 
     if( !pp_es ) /* no ES, or OOM */
@@ -701,7 +701,7 @@ libvlc_media_get_tracks_info( libvlc_media_t *p_md, libvlc_media_track_info_t **
     for( int i = 0; i < i_es; i++ )
     {
         libvlc_media_track_info_t *p_mes = *pp_es+i;
-        const es_format_t *p_es = p_input_item->es[i];
+        const input_item_track_info_t *p_es = p_input_item->tracks_info[i];
 
         p_mes->i_channels = p_mes->i_rate = 0;
         p_mes->i_width = p_mes->i_height = 0;
@@ -721,13 +721,13 @@ libvlc_media_get_tracks_info( libvlc_media_t *p_md, libvlc_media_track_info_t **
             break;
         case VIDEO_ES:
             p_mes->i_type = libvlc_track_video;
-            p_mes->i_height = p_es->video.i_height;
-            p_mes->i_width = p_es->video.i_width;
+            p_mes->i_height = p_es->u.video.i_height;
+            p_mes->i_width = p_es->u.video.i_width;
             break;
         case AUDIO_ES:
             p_mes->i_type = libvlc_track_audio;
-            p_mes->i_channels = p_es->audio.i_channels;
-            p_mes->i_rate = p_es->audio.i_rate;
+            p_mes->i_channels = p_es->u.audio.i_channels;
+            p_mes->i_rate = p_es->u.audio.i_rate;
             break;
         case SPU_ES:
             p_mes->i_type = libvlc_track_text;
diff --git a/src/input/es_out.c b/src/input/es_out.c
index 3062d8e..a1cbb0c 100644
--- a/src/input/es_out.c
+++ b/src/input/es_out.c
@@ -45,6 +45,7 @@
 #include "es_out.h"
 #include "event.h"
 #include "info.h"
+#include "item.h"
 
 #include "../stream_output/stream_output.h"
 
@@ -2811,6 +2812,9 @@ static void EsOutUpdateInfo( es_out_t *out, es_out_id_t *es, const es_format_t *
     const es_format_t *p_fmt_es = &es->fmt;
     lldiv_t         div;
 
+    /* Update the item tracks_info field */
+    input_item_UpdateTracksInfo(input_GetItem(p_input), fmt);
+
     /* Create category */
     char psz_cat[128];
     snprintf( psz_cat, sizeof(psz_cat),_("Stream %d"), es->i_meta_id );
diff --git a/src/input/item.c b/src/input/item.c
index 416ec19..4dcba24 100644
--- a/src/input/item.c
+++ b/src/input/item.c
@@ -99,6 +99,10 @@ static inline void input_item_Clean( input_item_t *p_i )
     }
     TAB_CLEAN( p_i->i_es, p_i->es );
 
+    for( i = 0; i < p_i->i_tracks_info; i++ )
+        free( p_i->tracks_info[i] );
+    TAB_CLEAN( p_i->i_tracks_info, p_i->tracks_info );
+
     for( i = 0; i < p_i->i_epg; i++ )
         vlc_epg_Delete( p_i->pp_epg[i] );
     TAB_CLEAN( p_i->i_epg, p_i->pp_epg );
@@ -1021,3 +1025,51 @@ void input_item_node_PostAndDelete( input_item_node_t *p_root )
 
   input_item_node_Delete( p_root );
 }
+
+/*
+ * Item tracks_info
+ */
+static input_item_track_info_t *track_info_new_from_format(const es_format_t *fmt)
+{
+    input_item_track_info_t *ret = malloc(sizeof(input_item_track_info_t));
+    ret->i_codec = fmt->i_codec;
+    ret->i_id = fmt->i_id;
+    ret->i_cat = fmt->i_cat;
+
+    ret->i_profile = fmt->i_profile;
+    ret->i_level = fmt->i_level;
+
+    ret->u.audio.i_channels = fmt->audio.i_channels;
+    ret->u.audio.i_rate = fmt->audio.i_rate;
+
+    ret->u.video.i_height = fmt->video.i_height;
+    ret->u.video.i_width = fmt->video.i_width;
+    return ret;
+}
+
+/* Called by es_out when a new Elementary Stream is added or updated. */
+void input_item_UpdateTracksInfo(input_item_t *item, const es_format_t *fmt)
+{
+    vlc_mutex_lock( &item->lock );
+    bool found = false;
+
+    input_item_track_info_t *track_info = track_info_new_from_format(fmt);
+
+    for( int i = 0; i < item->i_tracks_info; i++ )
+    {
+        if (item->tracks_info[i]->i_id == track_info->i_id)
+        {
+            /* We've found the right ES, replace it */
+            free(item->tracks_info[i]);
+            item->tracks_info[i] = track_info;
+            found = true;
+            break;
+        }
+    }
+    if( !found )
+    {
+        /* ES not found, create a new one. */
+        TAB_APPEND(item->i_tracks_info, item->tracks_info, track_info);
+    }
+    vlc_mutex_unlock( &item->lock );
+}
diff --git a/src/input/item.h b/src/input/item.h
index 971620a..ecf78cf 100644
--- a/src/input/item.h
+++ b/src/input/item.h
@@ -31,5 +31,6 @@
 #include "input_interface.h"
 
 void input_item_SetErrorWhenReading( input_item_t *p_i, bool b_error );
+void input_item_UpdateTracksInfo( input_item_t *item, const es_format_t *fmt );
 
 #endif
-- 
1.7.0

